// A very simple model for creating a kind of fractal foam
// with circles.  Keep track of all the circles you've added
// already, and keep trying to find new points where a circle
// of the current radius still fits without intersecting any
// other circles.  If you do manage to add a circle, or fail
// enough times at the current radius, make the
// radius slightly smaller and keep going.

// More information (and examples):
//  http://john-art.com/gallery5.html
//  http://archive.bridgesmathart.org/2014/bridges2014-79.pdf
//  http://www.paulbourke.net/texture_colour/randomtile/

class Circle
{
  float x;
  float y;
  float r;
  
  Circle( float x, float y, float r )
  {
    this.x = x;
    this.y = y;
    this.r = r;
  }
  
  boolean intersects( Circle other )
  {
    return dist( x, y, other.x, other.y ) <= (r+other.r);
  }
}

// All the circles we know about up to this point
Circle[] cs;

// The current radius of circle we're trying to place.
float radius;

boolean maybeAddCircle( Circle c )
{
  for ( int idx = 0; idx < cs.length; ++idx ) {
    if ( cs[idx].intersects( c ) ) {
      return false;
    }
  }

  cs = (Circle[])append( cs, c );
  return true;
}

void setup()
{
  size( 500, 500 );
  stroke( 0 );
  colorMode( HSB, 100 );
  radius = 80;
  cs = new Circle[0];
}

void draw()
{
  // Stop when the circles get small enough.
  if ( radius > 1 ) {
    // Try a thousand times to find a random circle location
    // that doesn't yet intersect other circles.  If you find 
    // it, draw it.  If you run out of tries, make the radius
    // slightly smaller.
    for ( int idx = 0; idx < 1000; ++idx ) {
      float x = random( width );
      float y = random( height );
      if ( maybeAddCircle( new Circle( x, y, radius ) ) ) {
        float t = norm( radius, 80, 0 );
        fill( lerp( 0, 75, sq( t ) ), 100, 100 );
        ellipse( x, y, 2*radius, 2*radius );
        return;
      }
    }

    radius = radius * 0.99;
  }
}

void keyPressed()
{
  cs = new Circle[0];
  radius = 80;
  background( 0, 0, 80 );
}